package at.ac.uibk.gitsearch.repository;

import static org.assertj.core.api.Assertions.assertThat;

import at.ac.uibk.gitsearch.IntegrationTest;
import at.ac.uibk.gitsearch.domain.ChatMessage;
import at.ac.uibk.gitsearch.domain.Review;
import at.ac.uibk.gitsearch.repository.jpa.ChatMessageRepository;
import at.ac.uibk.gitsearch.repository.jpa.ReviewRepository;

import java.util.Date;
import java.util.List;
import javax.transaction.Transactional;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;

@IntegrationTest
class ChatMessageRepositoryTest {

    @Autowired
    private ChatMessageRepository chatMessageRepository;

    @Autowired
    private ReviewRepository reviewRepository;

    @Test
    @Transactional
    void testFindByReview() {
        // Given
        Review review = new Review();
        review.setResource("TestResource"); // Setting the required 'resource' attribute
        reviewRepository.save(review);

        ChatMessage chatMessage1 = new ChatMessage();
        chatMessage1.setMessageText("Test message 1");
        chatMessage1.setTimestamp(new Date());
        chatMessage1.setReview(review);
        chatMessageRepository.save(chatMessage1);

        ChatMessage chatMessage2 = new ChatMessage();
        chatMessage2.setMessageText("Test message 2");
        chatMessage2.setTimestamp(new Date());
        chatMessage2.setReview(review);
        chatMessageRepository.save(chatMessage2);

        // When
        List<ChatMessage> chatMessages = chatMessageRepository.findByReview(review);

        // Then
        assertThat(chatMessages).hasSize(2);
    }

    @Test
    @Transactional
    void testFindByReviewOrderByTimestampAsc() {
        // Given
        Review review = new Review();
        review.setResource("TestResourceLater"); // Setting the required 'resource' attribute
        reviewRepository.save(review);

        ChatMessage chatMessage1 = new ChatMessage();
        chatMessage1.setMessageText("Test message later");
        chatMessage1.setTimestamp(new Date(new Date().getTime() + 3600000)); // 1 hour later
        chatMessage1.setReview(review);
        chatMessageRepository.save(chatMessage1);

        ChatMessage chatMessage2 = new ChatMessage();
        chatMessage2.setMessageText("Test message now");
        chatMessage2.setTimestamp(new Date());
        chatMessage2.setReview(review);
        chatMessageRepository.save(chatMessage2);

        // When
        List<ChatMessage> chatMessages = chatMessageRepository.findByReviewOrderByTimestampAsc(review);

        // Then
        assertThat(chatMessages.get(0)).isEqualTo(chatMessage2);
        assertThat(chatMessages.get(1)).isEqualTo(chatMessage1);
    }
}
